<?php
/*
Plugin Name: Star Rating for Reviews
Plugin URI: http://www.channel-ai.com/blog/plugins/star-rating/
Description: Insert inline rating stars within your posts based on the score you assign, supports outputting list of reviews sorted by date or scores.
Version: 0.3
Author: Yaosan Yeo
Author URI: http://www.channel-ai.com/blog/
*/

/*  Copyright 2006  Yaosan Yeo  (email : eyn@channel-ai.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// configurable variables (global variables)

	$sr_limitstar = 0;			// globally limit max number of stars, put 0 if choose not to use
	$sr_defaultstar = 5;			// default max score when not explicitly expressed, e.g. [rating:3.5] is evaluated as 3.5/$sr_defaultstar
	$sr_prefix = "<strong>Rating:</strong> ";					// prefix text before inserting star graphics, can leave blank if desired
	$sr_allprefix = "<strong>Overall Rating:</strong> ";	// prefix used for overall rating
	$sr_suffix = "";				// to close tags if you want to assign CSS class to star images (note: star images are hardcoded to have 0 padding, margin and border)
	$sr_ext = "png";				// file extension for star images
	$sr_usetext = 1;				// output plain text instead of star images in posts and feeds, options as below:
										// 0: images for posts and feeds;	1: images for posts, text for feeds;	2: text for all

	$sr_autometa = 0;				// option to insert meta key (custom field) automatically, put 1 to enable this feature
	$sr_metakey = 'rating';		// meta key for custom field, change this if there's conflict with other plugins

// end of configurable variables



// global variables for calculating cumulative average (do not touch)
	$sr_tscore = 0;
	$sr_counter = 0;



// sidebar functions

function sr_getreviews($limit=5, $orderby='date', $order='DESC', $usestar=0, $prefix='<li>', $suffix='</li>') {
	global $wpdb, $sr_metakey;

	$orderby = ($orderby == 'date') ? 'post_date' : 'meta_value+0';
	$order = ($order == 'DESC') ? 'DESC' : 'ASC';

	$sql = "SELECT id, post_title, meta_value";
	$sql .= " FROM  $wpdb->postmeta, $wpdb->posts";
	$sql .= " WHERE meta_key = '$sr_metakey'";
	$sql .= " AND post_id = id";
	$sql .= " AND post_status = 'publish'";
	$sql .= " ORDER BY $orderby";
	$sql .= " $order LIMIT $limit";

	$reviews = $wpdb->get_results($sql);
	
	// if no reviews can be found, output warning and exit
	if (empty($reviews)) {
		echo "Please make sure database writing has been enabled via plugin options and have at least one blog entry having its custom field properly set.\n";
	} else {
		foreach($reviews as $review) {
			$rating = $review->meta_value;
			$post_title = htmlentities(stripslashes($review->post_title));
			$permalink = get_permalink($review->id);

			// output image tags for stars instead of percentage if requested
			if ($usestar) {
				$rating_output = sr_usestar($rating, $usestar);
			// output percentage
			} else {
				$rating_output = $rating;
			}
	
			$output = $prefix . '<div class="sr-review"><a href="'. $permalink . '" title="' . $post_title . " - " . $rating . '%">' . $post_title . '</a></div> ';
			$output .= '<div class="sr-rating">' . $rating_output . "</div>";
			$output .= $suffix;

			echo $output . "\n";
		}
	}
}

function sr_getrandom($limit=5, $usestar=0, $prefix='<li>', $suffix='</li>') {
	global $wpdb, $sr_metakey;

	$sql = "SELECT id, post_title, meta_value";
	$sql .= " FROM  $wpdb->postmeta, $wpdb->posts";
	$sql .= " WHERE meta_key = '$sr_metakey'";
	$sql .= " AND post_id = id";
	$sql .= " AND post_status = 'publish'";

	$reviews = $wpdb->get_results($sql);
	
	// if no reviews can be found, output warning and exit
	if (empty($reviews)) {
		echo "Please make sure database writing has been enabled via plugin options and have at least one blog entry having its custom field properly set.\n";
	} else {
		// randomize the returned 2D array
		$shuffles = array_rand($reviews, $limit);
		$review = array();

		foreach($shuffles as $shuffle) {
			$review = $reviews[$shuffle];

			$rating = $review->meta_value;
			$post_title = htmlentities(stripslashes($review->post_title));
			$permalink = get_permalink($review->id);

			// output image tags for stars instead of percentage if requested
			if ($usestar) {
				$rating_output = sr_usestar($rating, $usestar);
			// output percentage
			} else {
				$rating_output = $rating;
			}

			$output = $prefix . '<div class="sr-review"><a href="'. $permalink . '" title="' . $post_title . " - " . $rating . '%">' . $post_title . '</a></div> ';
			$output .= '<div class="sr-rating">' . $rating_output . "</div>";
			$output .= $suffix;

			echo $output . "\n";
		}
	}
}



// single post functions

function sr_getsingle($post_id, $usestar=0, $prefix='', $suffix='',$size=0) {
	global $wpdb, $sr_metakey;

	$sql = "SELECT meta_value";
	$sql .= " FROM  $wpdb->postmeta";
	$sql .= " WHERE meta_key = '$sr_metakey'";
	$sql .= " AND post_id = '$post_id'";

	$rating = $wpdb->get_var($sql);

	// jump out of function if no rating data can be found for this single post
	if (empty($rating))
		return 0;

	// output image tags for stars instead of percentage if requested
	if ($usestar) {
		$rating_output = sr_usestar($rating, $usestar, $size);
	// output percentage
	} else {
		$rating_output = $rating;
	}

	$output = $prefix . $rating_output . $suffix;

	echo $output . "\n";
}



// common function for sidebar and single post function
function sr_usestar($rating, $star_count, $big=0) {
	global $sr_ext;

	$star = $rating / 100 * $star_count;

	if ($big)
		$path = trailingslashit(get_settings('siteurl')) . 'wp-content/plugins/star-rating/';
	else
		$path = trailingslashit(get_settings('siteurl')) . 'wp-content/plugins/star-rating/tiny-';

	// check if half star occurs
	// e.g. [3.75 , 4.25) = 4 stars; [4.25 , 4.75) = 4.5 stars
	$halfstar = "";
	$star = round( $star * 2 );

	if ( $star % 2 ) {
		$halfstar = '<img src="' . $path . 'halfstar.' . $sr_ext . '" alt="&frac12;" />';
		$star = floor( $star / 2 );
		$blankstar = $star_count - $star - 1;
	} else 	{
		$star = $star / 2;
		$blankstar = $star_count - $star;
	}

	// finally, generate html for rating stars
	$output = str_repeat ('<img src="' . $path . 'star.' . $sr_ext . '" alt="&#9733;" />', $star) . $halfstar . str_repeat('<img src="' . $path . 'blankstar.' . $sr_ext . '" alt="&#9734;" />', $blankstar);

	return $output;
}



// post formatting functions

function sr_addstar($content)
{
	$content = preg_replace_callback( "/(?<!`)\[rating:(([^]]+))]/i","sr_genstar", $content );
	$content = preg_replace( "/`(\[rating:(?:[^]]+)])`?/i", "$1", $content );	//escape rating tags by using `[rating:]`

	return $content;
}

function sr_genstar($matches) {
	global $sr_limitstar, $sr_defaultstar, $sr_prefix, $sr_allprefix, $sr_suffix, $sr_ext, $sr_usetext, $sr_tscore, $sr_counter;

	list($score, $maxscore) = explode("/", $matches[2]);

	// check if we should get overall rating
	if ( strncasecmp(trim($score), "overall", 7) == 0 ) {
		$percent = $sr_tscore / $sr_counter;

		if ($maxscore)
			$maxstar = $maxscore;
		else
			$maxstar = $sr_defaultstar;

		$prefix = $sr_allprefix;

		// clear cummulative variables for cases where multiple overall rating is required within single post
		$sr_tscore = 0;
		$sr_counter = 0;
	}

	// check if it is the escaping rating tag to prevent autometa
	elseif ( strncasecmp(trim($score), "none", 4) == 0 ) {
		return "";
	}

	// if not overall, calculate rating based on score assigned
	else {
		if ($maxscore) {
			// limit max number of stars to 20
			$maxstar = ($maxscore <= 20) ? $maxscore : $sr_defaultstar;
		}
		else {
			$maxscore = $sr_defaultstar;
			$maxstar = $sr_defaultstar;
		}

		// check if we should limit the global max number of stars
		if ($sr_limitstar){
			$maxstar = $sr_limitstar;
		}

		$percent = $score / $maxscore;
		$sr_counter++;
		$sr_tscore += $percent;
		$prefix = $sr_prefix;
	}

	$star = $percent * $maxstar;

	$path = trailingslashit(get_settings('siteurl')) . 'wp-content/plugins/star-rating/';

	// check if half star occurs
	// e.g. [3.75 , 4.25) = 4 stars; [4.25 , 4.75) = 4.5 stars
	$halfstar = "";
	$star = round( $star * 2 );

	if ( $star % 2 ) {
		$halfstar = '<img src="' . $path . 'halfstar.' . $sr_ext . '" alt="&frac12;" style="border:0;margin:0;padding:0" />';
		$star = floor( $star / 2 );
		$blankstar = $maxstar - $star - 1;
	}
	else
	{
		$star = $star / 2;
		$blankstar = $maxstar - $star;
	}

	// finally, generate html for rating stars
	$code = $prefix . str_repeat ('<img src="' . $path . 'star.' . $sr_ext . '" alt="&#9733;" style="border:0;margin:0;padding:0" />', $star) . $halfstar . str_repeat('<img src="' . $path . 'blankstar.' . $sr_ext . '" alt="&#9734;" style="border:0;margin:0;padding:0" />', $blankstar);
	$code .= $sr_suffix;

	// generate alternative plain text output
	if ($halfstar != "")
		$star += 0.5;

	$textcode = $prefix . $star . " out of " . round($maxstar) . " stars" . $sr_suffix;

	// output code based on options
	if ( $sr_usetext == 1 && is_feed() )
		return $textcode;
	elseif ( $sr_usetext == 2 )
		return $textcode;
	else
		return $code;
}

add_filter('the_content', 'sr_addstar');
add_filter('the_excerpt', 'sr_addstar');



// post editing / meta values functions

function sr_addratings($post_id) {
	global $wpdb, $sr_tscore, $sr_counter, $sr_autometa, $sr_metakey;

	// jump out of function if auto meta is not enabled
	if (!$sr_autometa) {
		return $post_id;
	}

	$content = $wpdb->get_var("SELECT post_content
							FROM $wpdb->posts
							WHERE ID = '$post_id'");

	// parse post content for rating tags, and store rating information in global variables
	$content = preg_replace_callback( "/(?<!`)\[rating:(([^]]+))]/i", "sr_genratings", $content );

	// calculate overall ratings
	if ($sr_tscore > 0) {
		$rating = round( $sr_tscore / $sr_counter * 100 , 1 );	// in percentage
	}
	else {
		return $post_id;	// jump out of function if [rating:none] is found or no [rating:] tags at all
	}

	// read rating custom field from database
	$rating_meta = $wpdb->get_var("SELECT meta_value
							FROM $wpdb->postmeta
							WHERE post_id = '$post_id'
							AND meta_key = '$sr_metakey'");

	// if already exist, update the value with new ratings
	if (!empty($rating_meta)) {
		$results=  $wpdb->query("UPDATE $wpdb->postmeta
								SET meta_value = '$rating'
								WHERE post_id = '$post_id'
								AND meta_key = '$sr_metakey'");
	// if not exist, write rating custom field to database
	} else {
		$results = $wpdb->query("INSERT INTO $wpdb->postmeta (post_id,meta_key,meta_value)
								VALUES ('$post_id', '$sr_metakey', '$rating')");
	}

	return $post_id;
}

function sr_genratings($matches) {
	global $sr_defaultstar, $sr_tscore, $sr_counter;

	list($score, $maxscore) = explode("/", $matches[2]);

	if (!$maxscore) {
			$maxscore = $sr_defaultstar;
	}

	// if not overall rating, add to cumulative
	if ( !(strncasecmp(trim($score), "overall", 7) == 0) ) {
		$percent = $score / $maxscore;
		$sr_counter++;
		$sr_tscore += $percent;
	}

	// disable autometa if [rating:none] is found
	if ( strncasecmp(trim($score), "none", 4) == 0) {
		$sr_tscore = -99999;
	}

	return 0;
}

add_action('save_post', 'sr_addratings')
?>
